<?php
namespace Sharplogicians\Packages\Block\Adminhtml\Package;

class Grid extends \Magento\Backend\Block\Widget\Grid\Extended
{
    /**
     * @var \Magento\Framework\Module\Manager
     */
    protected $moduleManager;

    /**
     * @var \Sharplogicians\Packages\Model\packageFactory
     */
    protected $_packageFactory;

    /**
     * @var \Sharplogicians\Packages\Model\Status
     */
    protected $_status;

    /**
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Backend\Helper\Data $backendHelper
     * @param \Sharplogicians\Packages\Model\packageFactory $packageFactory
     * @param \Sharplogicians\Packages\Model\Status $status
     * @param \Magento\Framework\Module\Manager $moduleManager
     * @param array $data
     *
     * @SuppressWarnings(PHPMD.ExcessiveParameterList)
     */
    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Backend\Helper\Data $backendHelper,
        \Sharplogicians\Packages\Model\PackageFactory $PackageFactory,
        \Sharplogicians\Packages\Model\Status $status,
        \Magento\Framework\Module\Manager $moduleManager,
        array $data = []
    ) {
        $this->_packageFactory = $PackageFactory;
        $this->_status = $status;
        $this->moduleManager = $moduleManager;
        parent::__construct($context, $backendHelper, $data);
    }

    /**
     * @return void
     */
    protected function _construct()
    {
        parent::_construct();
        $this->setId('postGrid');
        $this->setDefaultSort('id');
        $this->setDefaultDir('DESC');
        $this->setSaveParametersInSession(true);
        $this->setUseAjax(false);
        $this->setVarNameFilter('post_filter');
    }

    /**
     * @return $this
     */
    protected function _prepareCollection()
    {
        $collection = $this->_packageFactory->create()->getCollection();
        $this->setCollection($collection);

        parent::_prepareCollection();

        return $this;
    }

    /**
     * @return $this
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     */
    protected function _prepareColumns()
    {
        $this->addColumn(
            'id',
            [
                'header' => __('ID'),
                'type' => 'number',
                'index' => 'id',
                'header_css_class' => 'col-id',
                'column_css_class' => 'col-id'
            ]
        );
		
        $this->addColumn(
            'product_skus',
            [
                'header' => __('Products'),
                'index' => 'product_skus',
            ]
        );
        
        $this->addColumn(
            'name',
            [
                'header' => __('Package Name'),
                'index' => 'name',
            ]
        );
        
        $this->addColumn(
            'weight',
            [
                'header' => __('Package Weight'),
                'index' => 'weight',
            ]
        );
        
        $this->addColumn(
            'length',
            [
                'header' => __('Package Length'),
                'index' => 'length',
            ]
        );
        
        $this->addColumn(
            'width',
            [
                'header' => __('Package Width'),
                'index' => 'width',
            ]
        );
        
        $this->addColumn(
            'height',
            [
                'header' => __('Package Height'),
                'index' => 'height',
            ]
        );

        $block = $this->getLayout()->getBlock('grid.bottom.links');
        if ($block) {
            $this->setChild('grid.bottom.links', $block);
        }

        return parent::_prepareColumns();
    }

	
    /**
     * @return $this
     */
    protected function _prepareMassaction()
    {

        $this->setMassactionIdField('id');
        //$this->getMassactionBlock()->setTemplate('Sharplogicians_Packages::package/grid/massaction_extended.phtml');
        $this->getMassactionBlock()->setFormFieldName('package');

        $this->getMassactionBlock()->addItem(
            'delete',
            [
                'label' => __('Delete'),
                'url' => $this->getUrl('packages/*/massDelete'),
                'confirm' => __('Are you sure?')
            ]
        );

        return $this;
    }
		

    /**
     * @return string
     */
    public function getGridUrl()
    {
        return $this->getUrl('packages/*/index', ['_current' => true]);
    }

    /**
     * @param \Sharplogicians\Packages\Model\package|\Magento\Framework\Object $row
     * @return string
     */
    public function getRowUrl($row)
    {
		
        return $this->getUrl(
            'packages/*/edit',
            ['id' => $row->getId()]
        );
		
    }

	

}