<?php
namespace Sharplogicians\Packages\Plugin;

use Magento\Quote\Model\Quote\Address\RateRequest;
use Magento\Ups\Model\Carrier;
use Magento\Framework\DataObject;

class UpsCarrier
{
    /**
     * @var \Sharplogicians\Packages\Model\Helper
     */
    private $helper;

    public function __construct(\Sharplogicians\Packages\Model\Helper $helper)
    {
        $this->helper = $helper;
    }
    /**
     * @param Carrier $subject
     * @param \Magento\Usps\Model\Carrier $result
     * @param RateRequest $request
     */
    public function aroundSetRequest(Carrier $subject, callable $proceed, RateRequest $request)
    {
        $writer = new \Zend_Log_Writer_Stream(BP . '/var/log/upds.log');
        $logger = new \Zend_Log();
        $logger->addWriter($writer);
        $logger->info(__METHOD__);
        
        $packages = [];
        $predefined_packages = [];
        $totalWeight = 0;
        $predefinedPackages = [];
        $predefined_package_count = 0;

        $this->_request = $request;

        $rowRequest = new DataObject();

        if ($request->getLimitMethod()) {
            $rowRequest->setAction($this->configHelper->getCode('action', 'single'));
            $rowRequest->setProduct($request->getLimitMethod());
        } else {
            $rowRequest->setAction($this->configHelper->getCode('action', 'all'));
            $rowRequest->setProduct('GND' . $this->getConfigData('dest_type'));
        }

        if ($request->getUpsPickup()) {
            $pickup = $request->getUpsPickup();
        } else {
            $pickup = $this->getConfigData('pickup');
        }
        $rowRequest->setPickup($this->configHelper->getCode('pickup', $pickup));

        if ($request->getUpsContainer()) {
            $container = $request->getUpsContainer();
        } else {
            $container = $this->getConfigData('container');
        }
        $rowRequest->setContainer($this->configHelper->getCode('container', $container));

        if ($request->getUpsDestType()) {
            $destType = $request->getUpsDestType();
        } else {
            $destType = $this->getConfigData('dest_type');
        }
        $rowRequest->setDestType($this->configHelper->getCode('dest_type', $destType));

        if ($request->getOrigCountry()) {
            $origCountry = $request->getOrigCountry();
        } else {
            $origCountry = $this->_scopeConfig->getValue(
                OrderShipment::XML_PATH_STORE_COUNTRY_ID,
                ScopeInterface::SCOPE_STORE,
                $request->getStoreId()
            );
        }

        $rowRequest->setOrigCountry($this->_countryFactory->create()->load($origCountry)->getData('iso2_code'));

        if ($request->getOrigRegionCode()) {
            $origRegionCode = $request->getOrigRegionCode();
        } else {
            $origRegionCode = $this->_scopeConfig->getValue(
                OrderShipment::XML_PATH_STORE_REGION_ID,
                ScopeInterface::SCOPE_STORE,
                $request->getStoreId()
            );
        }
        if (is_numeric($origRegionCode)) {
            $origRegionCode = $this->_regionFactory->create()->load($origRegionCode)->getCode();
        }
        $rowRequest->setOrigRegionCode($origRegionCode);

        if ($request->getOrigPostcode()) {
            $rowRequest->setOrigPostal($request->getOrigPostcode());
        } else {
            $rowRequest->setOrigPostal(
                $this->_scopeConfig->getValue(
                    OrderShipment::XML_PATH_STORE_ZIP,
                    ScopeInterface::SCOPE_STORE,
                    $request->getStoreId()
                )
            );
        }

        if ($request->getOrigCity()) {
            $rowRequest->setOrigCity($request->getOrigCity());
        } else {
            $rowRequest->setOrigCity(
                $this->_scopeConfig->getValue(
                    OrderShipment::XML_PATH_STORE_CITY,
                    ScopeInterface::SCOPE_STORE,
                    $request->getStoreId()
                )
            );
        }

        if ($request->getDestCountryId()) {
            $destCountry = $request->getDestCountryId();
        } else {
            $destCountry = self::USA_COUNTRY_ID;
        }

        $country = $this->_countryFactory->create()->load($destCountry);
        $rowRequest->setDestCountry($country->getData('iso2_code') ?: $destCountry);

        $rowRequest->setDestRegionCode($request->getDestRegionCode());

        if ($request->getDestPostcode()) {
            $rowRequest->setDestPostal($request->getDestPostcode());
        }

        $rowRequest->setOrigCountry(
            $this->getNormalizedCountryCode(
                $rowRequest->getOrigCountry(),
                $rowRequest->getOrigRegionCode(),
                $rowRequest->getOrigPostal()
            )
        );

        $rowRequest->setDestCountry(
            $this->getNormalizedCountryCode(
                $rowRequest->getDestCountry(),
                $rowRequest->getDestRegionCode(),
                $rowRequest->getDestPostal()
            )
        );

        if ($request->getFreeMethodWeight() != $request->getPackageWeight()) {
            $rowRequest->setFreeMethodWeight($request->getFreeMethodWeight());
        }

        foreach ($request->getAllItems() as $item) {
            if (!$item->getProduct() || !$item->getProduct()->getWeight()) {
                continue;
            }

            $product = $item->getProduct();
            $qty = $item->getQty();
            $productWeight = $product->getWeight() * $qty;            

            $predefinedPackages = $this->getPredefinedPackage($product);
            if($predefinedPackages){
                $totalWeight += $productWeight;
                foreach($predefinedPackages as $predefined) {
                    if ($predefined) {
                        for ($i = 0; $i < $qty; $i++) {
                            $predefined_packages[] = [
                                'length' => $predefined->getLength(),
                                'width'  => $predefined->getWidth(),
                                'height' => $predefined->getHeight(),
                                'weight' => $predefined->getWeight(),
                            ];
                        }               
                        
                        $predefined_package_count += 1;
                    }
                }
            }          
        }

        

        /*$packages = $this->createPackages(
            (float) $request->getPackageWeight(),
            (array) $request->getPackages()
        );*/

        $logger->info(print_r($predefined_packages));

        $packages = array_merge($predefined_packages, $packages);

        $logger->info(print_r($packages));

        $rowRequest->setPackages(
            $packages
        );

        $rowRequest->setWeight($this->_getCorrectWeight($request->getPackageWeight()));
        $rowRequest->setValue($request->getPackageValue());
        $rowRequest->setValueWithDiscount($request->getPackageValueWithDiscount());

        if ($request->getUpsUnitMeasure()) {
            $unit = $request->getUpsUnitMeasure();
        } else {
            $unit = $this->getConfigData('unit_of_measure');
        }
        $rowRequest->setUnitMeasure($unit);
        $rowRequest->setIsReturn($request->getIsReturn());
        $rowRequest->setBaseSubtotalInclTax($request->getBaseSubtotalInclTax());

        $this->_rawRequest = $rowRequest;

        return $this;
    }

    public function createCustomPackages($items, $maxWeight)
    {
        $packages = [];
        $predefined_packages = [];
        $totalWeight = 0;
        $predefinedPackages = [];
        $predefined_package_count = 0;

        foreach ($items as $item) {
            if (!$item->getProduct() || !$item->getProduct()->getWeight()) {
                continue;
            }

            $product = $item->getProduct();
            $qty = $item->getQty();
            $productWeight = $product->getWeight() * $qty;

            

            $predefinedPackages = $this->getPredefinedPackage($product);
            if($predefinedPackages){
                foreach($predefinedPackages as $predefined) {
                    if ($predefined) {
                        for ($i = 0; $i < $qty; $i++) {
                            $predefined_packages[] = [
                                'length' => $predefined->getLength(),
                                'width'  => $predefined->getWidth(),
                                'height' => $predefined->getHeight(),
                                'weight' => $predefined->getWeight(),
                            ];
                        }               
                        
                        $predefined_package_count += 1;
                    }
                }
            } else {
                $totalWeight += $productWeight;
            }            
        }

        $packages = array_merge($predefinedpackages, $packages);

        $this->logger->info('Packages created: ' . count($packages));

        return $packages;
    }

    private function getPredefinedPackage($product)
    {
        if($sku = $product->getSku()) {
            return $this->_packageFactory->create()->getCollection()->addFieldToFilter('product_skus', $sku);
        }

        return null;
    }
}
